; docformat = 'rst'
;+
; This is an example program to demonstrate how to create a lidar range plot
; with Coyote Graphics routines.
;
; :Categories:
;    Graphics
;    
; :Examples:
;    Save the program as "lidar_range_plot.pro" and run it like this::
;       IDL> .RUN lidar_range_plot
;       
; :Author:
;    FANNING SOFTWARE CONSULTING::
;       David W. Fanning 
;       1645 Sheely Drive
;       Fort Collins, CO 80526 USA
;       Phone: 970-221-0438
;       E-mail: david@idlcoyote.com
;       Coyote's Guide to IDL Programming: http://www.idlcoyote.com
;
; :History:
;     Change History::
;        Written, 23 January 2013 by David W. Fanning.
;
; :Copyright:
;     Copyright (c) 2013, Fanning Software Consulting, Inc.
;-
PRO Lidar_Range_Plot, QUALITY=quality
 
   ; Set up variables for the plot. Normally, these values would be 
   ; passed into the program as positional and keyword parameters.
   ; Restore the lidar range data in the structure named lidarData.
   Restore, File='lidardata.sav'
   Help, lidarData, /Structure
   xtitle = 'Range (km)'
   ytitle = 'Range (km)'
   
   ; Set the quality of the rendering: 1 - low, 2 - medium, 3 -high, 4-highest
   IF N_Elements(quality) EQ 0 THEN quality = 2
   CASE quality of
      1: ratio = 100
      2: ratio = 50
      3: ratio = 10
      4: ratio = 1
   ENDCASE   
       
   ; We want the plot to be drawn in thicker lines.
   thick = (!D.Name EQ 'PS') ? 6 : 2
   
   ; Scale the radius of the beam into the beam range. Find the 0.0 point,
   ; which is where we want to start the beam plot.
   r = cgScaleVector(Findgen(lidarData.numBeamPts), lidarData.range[0], lidarData.range[1])
   index = Value_Locate(r, 0.0)
   IF r[index] LT 0.0 THEN index = index + 1
   r = r[index:*]
   beams = lidarData.beams[index:*, *]
   
   ; There is LOTS of data here. We don't need it all for display.
   ; Prune it by the plot quality ratio.
   numBeamPts = Round((Size(beams, /Dimensions))[0] / ratio)
   angdiff = Median(Abs(lidarData.angles - Shift(lidarData.angles, 1))) / 2.0
   angdiff = angdiff * 1.025
   beams = Congrid(beams, numBeamPts, lidarData.numBeams)
   r = Congrid(r, numBeamPts)
   
   ; Set up the display colors.
   cgLoadCT, 26, /Brewer, /Reverse, NColors=64, Bottom=2
   TVLCT, cgColor('white', /Triple), 1
   TVLCT, cgColor('charcoal', /Triple), 66
   
   ; Set up scaling thresholds.
   minThresh = -5 ; dB
   maxThresh = 5 ; dB
   
   ; Scale data and locate out-of-bound values.
   beamsScaled = BytScl(beams, TOP=63, MIN=minThresh, MAX=maxThresh) + 2B ; Scaled from 2-65
   lowValues = Where(beams LT minThresh, lowcount)
   IF lowcount GT 0 THEN beamsScaled[lowValues] = 1B
   highValues = Where(beams GT maxThresh, highcount)
   IF highcount GT 0 THEN beamsScaled[highValues] = 66B
   
   ; Convert plot ranges to rectangular coordinate space.
   prange = Transpose([[lidarData.angles, lidarData.angles], $
      [Replicate(Max(r), lidarData.numBeams), Replicate(Min(r) , lidarData.numBeams)]])
   xyrange = CV_Coord(FROM_POLAR=prange, /DEGREES, /TO_RECT)
   xrange = [Min(xyrange[0,*]), Max(xyrange[0,*])]
   xdiff = xrange[1] - xrange[0]
   xrange = xrange + [-0.1*xdiff, 0.1*xdiff]
   yrange = [Min(xyrange[1,*]), Max(xyrange[1,*])]
   ydiff = yrange[1] - yrange[0]
   yrange = yrange + [-0.05*ydiff, 0.05*ydiff]
   
   ; The vectors need to be rotated 90 degrees for display.
   angleRange = lidarData.angleRange
   angleRange = ((360+90) - angleRange) MOD 360
   
   ; Get a smooth set of angles over the angle range.
   angles = cgScaleVector(Findgen(numBeamPts), angleRange[0], angleRange[1])

   ; Maintain the aspect ratio of the data range in the plot.
   position = Aspect(Float((yrange[1]-yrange[0]))/(xrange[1]-xrange[0]), $
      Margin=0.1)
      
   ; Shift the position to the left to allow for a color bar.
   position[[0,2]] = position[[0,2]] - 0.03
   
   ; Open a display window.
   cgDisplay, 675, 600

   ; Set up a data coordinate system.
   xs = cgNormalize(xrange, Position=[position[0], position[2]])
   ys = cgNormalize(yrange, Position=[position[1], position[3]])
  
   !X.S = xs
   !Y.S = ys
   !X.Window = [position[0], position[2]]
   !Y.Window = [position[1], position[3]]
  
   ; Set the clipping rectangle.
   d = Convert_Coord(!X.Window, !Y.Window, /Normal, /To_Device)
   !P.Clip = [d[0,0], d[1,0], d[0,1], d[1,1], 0, 0]
  
   ; Set the calculated ranges.
   !X.CRange = xrange
   !Y.CRange = yrange
   
   ; Draw each data beam as a series of filled polygons.
   FOR j=0,lidarData.numBeams-1 DO BEGIN
      a = Replicate(lidarData.angles[j], numBeamPts)
      FOR k=0,numBeamPts-2 DO BEGIN
         ang = [a[k]-angdiff, a[k], a[k]+angdiff, a[k]+angdiff, a[k], a[k]-angdiff, a[k]-angdiff]
         rad = [    r[k],     r[k],    r[k],         r[k+1],     r[k+1],  r[k+1],        r[k]]
         polar = Transpose([[ang], [rad]])
         xy = CV_Coord(FROM_POLAR=polar, /DEGREES, /TO_RECT)
         x = Reform(xy[0,*])
         y = Reform(xy[1,*])
         cgColorFill, x, y, COLOR=beamsScaled[k,j], NOCLIP=0
      ENDFOR
    ENDFOR
   
    ; Draw a grid on the plot.
   cgPlot, [0,1], /NODATA, XSTYLE=1, YSTYLE=1, $
      POSITION=position, XTICK_GET=xticks, YTICK_GET=yticks, $
      XRANGE=xrange, YRANGE=yrange, AXISCOLOR='gray', $
      XTICKFORMAT='(A1)', YTICKFORMAT='(A1)', XTICKLEN=1, YTICKLEN=1, $
      XGRIDSTYLE=2, YGRIDSTYLE=2, /NOERASE, THICK=thick
   ytickname = String(Abs(yticks), Format='(F0.1)')
   xtickname = String(Abs(xticks), Format='(F0.1)')
   cgPlot, [0,1], /NODATA, XSTYLE=1, YSTYLE=1, XTICKNAME=xtickname, $
      YTICKNAME=ytickname, POSITION=position, NOERASE=1, $
      XRANGE=xrange, YRANGE=yrange, $
      XTITLE=xtitle, YTITLE=ytitle, XTICKLEN=0.025, YTICKLEN=0.025, THICK=thick

   ; Draw the colorbar.
   cgColorbar, NColors=64, Bottom=2, OOB_Low=1B, OOB_High=66B, $
       Range=[minThresh, maxThresh], /Vertical, Divisions=10, $
       Position=[0.91, position[1], 0.94, position[3]]
       
   ; Draw the range markers.
   FOR j=Round(Min(r)), Round(Max(r)), 2 DO BEGIN
      IF j GT Max(r) THEN BREAK
      angles = cgScaleVector(Findgen(100), Min(lidarData.angles), Max(lidarData.angles))
      polarCoords = Transpose([[angles],[Replicate(j,100)]])
      cc = CV_Coord(FROM_POLAR=polarCoords, /DEGREES, /TO_RECT)
      cgPlotS, cc[0,*], cc[1,*], COLOR='red', $
         THICK=thick, LINESTYLE=2, NOCLIP=0
      IF cc[0,0] LE cc[0,99] THEN BEGIN
         cgText, cc[0,99]+0.11, cc[1,99]-0.125, StrTrim(j,2), $
            COLOR='red' , ALIGNMENT=0.0, NOCLIP=0, CHARSIZE=1.5 
      ENDIF ELSE BEGIN
         cgText, cc[0,0]-0.125, cc[1,0]+0.11, StrTrim(j,2), $
            COLOR='red', ALIGNMENT=1.0, NOCLIP=0, CHARSIZE=1.5
      ENDELSE
   ENDFOR
   
   ; Other labels.
   cgText, 0.95, 0.495, /Normal, 'dB'
   
   scantimeText = 'Start: ' + lidarData.hhmmss[0] + ' UTC'
   angleText = 'Elevation: ' + String(lidarData.fixedAngle, FORMAT='(F7.2)') + '$\deg$'
   
   cgText, position[0], position[3] + (2.5*0.025), /NORMAL, scantimeText
   cgText, position[2], position[3] + (2.5*0.025), /NORMAL, angleText, ALIGNMENT=1
   cgText, position[0], position[3] + 0.025, /NORMAL, lidarData.scanType + ' Scan ' + $
     StrTrim(lidarData.scanNumber,2) + ' filtered'
    
   year = Fix(StrMid(lidarData.yyyymmdd, 0, 4))
   month = Fix(StrMid(lidarData.yyyymmdd, 4, 2))
   day = Fix (StrMid(lidarData.yyyymmdd, 6, 2))
   date = String(day, Format='(I0)') + ' ' + $
     cgMonths(month, /ABBREVIATION, /FIRSTLETTER) + ' ' + $
     String(year, Format='(I4)')
   cgText, position[2], position[3] + 0.025, /NORMAL, date, $
     ALIGNMENT=1
END ;*****************************************************************

; This main program shows how to call the program and produce
; various types of output.

  ; Display the plot in a graphics window.
  Lidar_Range_Plot
  
  ; Display the plot in a resizeable graphics window.
  cgWindow, 'Lidar_Range_Plot', WBackground='White', WXSize=675, WYSize=600, $
       WAspect=600./675, WTitle='Lidar Range Plot in a Resizeable Graphics Window'
  
  ; Create a PostScript file.
  cgPS_Open, 'lidar_range_plot.ps'
  Lidar_Range_Plot
  cgPS_Close
  
  ; Create a PNG file with a width of 600 pixels.
  cgPS2Raster, 'lidar_range_plot.ps', /PNG, Width=600

END